;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x33b17e360d5c5aafc065a40b54b48a70 . "./code.cg2js/cg-acache.fasl")
(#x4615e6dcaab13a5458b610913ecf9286 . "./code.cg2js/cg-base.fasl")
(#x72d9e23308911015120a81eba5457e68 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x0eb5fcf073ce3386dc818d1c0b17f218 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x8506cc55731106de82544d631b73f5f7 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xdc5c9583785a7e5b34dc36bf1ed754ca . "./code.cg2js/cg-button.fasl")
(#x403f90e842bf14272d056734a6a847f2 . "./code.cg2js/cg-calendar.fasl")
(#x2b4027ee32563cc9a9ea4bcc15169d55 . "./code.cg2js/cg-caret.fasl")
(#xcc467237bbe29ae4daa25a5ed26d26e7 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x524de6533b0dbba41417da2cc335c02c . "./code.cg2js/cg-chart-widget.fasl")
(#x897575bbd5488ec537ad1e0c5389133c . "./code.cg2js/cg-check-box.fasl")
(#x11b4cb8bcff3738cd4398a9521772550 . "./code.cg2js/cg-choice-list.fasl")
(#x9bcbdc28806fd79e83e2d3dd8e0f806d . "./code.cg2js/cg-choose-printer.fasl")
(#xa5dcba16dffebe51329f34c96d25a51d . "./code.cg2js/cg-class-grid.fasl")
(#x5c2c3650651761f996b495fe3421fd8b . "./code.cg2js/cg-class-slot-grid.fasl")
(#x9a21e861b28589f5ee2290abfa9ca8af . "./code.cg2js/cg-class-support.fasl")
(#x6fec52e3e2c3a97e281078084a4fd360 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xb10da81bdff65448cf70b35b83a7b101 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x58869bb4ffc9043db5ca77c042da0e0f . "./code.cg2js/cg-clipboard.fasl")
(#x1e56c0af6710f2843cf79278c3dd78d5 . "./code.cg2js/cg-color-dialog.fasl")
(#x62b9e17c2f1bf1c14631ee4b8d7609dd . "./code.cg2js/cg-combo-box.fasl")
(#x96ab0f31d8134fa478e1e83b69de3d52 . "./code.cg2js/cg-common-control.fasl")
(#x6a0883c85b1335f39d4d81d8c51bc420 . "./code.cg2js/cg-comtab.fasl")
(#x93f8d8234d654144f4b47052fd6d994c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xe84b64665c4e7872b5ef68449ac3f3dc . "./code.cg2js/cg-curve.fasl")
(#x0d6b53351b1d92f90266fe17dc41a32a . "./code.cg2js/cg-dde-utils.fasl")
(#xe32d37ff65b10db0471bc4698e33cde1 . "./code.cg2js/cg-dev.fasl")
(#x22f7063cfdfea718eff6a2342920abc2 . "./code.cg2js/cg-dialog-item.fasl")
(#x69201874a13ae8e4230700f89885c870 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x5e1b8a31c5db45e532cce7dd5ee7e841 . "./code.cg2js/cg-directory-dialog.fasl")
(#x1abd4eb9d8b1c8572980f105d9f9fe93 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x517042b3f6288648730a544c55cebd04 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x33715d16b8108d20e843767200ebd351 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa6488b868ce8079e13014d6128332f7c . "./code.cg2js/cg-drawable.fasl")
(#x93356ef9113bf2d0332699b6b8014c8f . "./code.cg2js/cg-dropping-outline.fasl")
(#x826738fbe752d96b76eb5df276f16a85 . "./code.cg2js/cg-edit-in-place.fasl")
(#xd7a532f31e2b233580e4e62cc5a72cfd . "./code.cg2js/cg-editable-text.fasl")
(#x5321d564eca233d9ff5ebef9f89d674d . "./code.cg2js/cg-file-dialog.fasl")
(#x8cd6f2d1e86711aff37d24b2c331eaa3 . "./code.cg2js/cg-fill-texture.fasl")
(#x823477a6655ebca253c0a329fd2cf381 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xc99ba21225760756c95e1eb9e9f83a91 . "./code.cg2js/cg-font-dialog.fasl")
(#x3399edd3ed48cec3925b54a8106e7ea8 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xbb38e887e6a6cd32a13dd5a03635b933 . "./code.cg2js/cg-get-pixmap.fasl")
(#x49fef1b605595ecf55411e54b60f4a2c . "./code.cg2js/cg-get-position.fasl")
(#x78941166f8f6e5fa5554a154c2dc22a8 . "./code.cg2js/cg-graphics-context.fasl")
(#xc1503b2afbcb47c411ef13a06b584f80 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x104b3bb4f30448ad480ab124795a073d . "./code.cg2js/cg-grid-widget.fasl")
(#x7458fcffce5ff33f0b7420edfd9c4179 . "./code.cg2js/cg-group-box.fasl")
(#xaee9a7a249473fe3d8e22e516ddd2341 . "./code.cg2js/cg-header-control.fasl")
(#xca6295f93169af4806e8feb53d9c9c1e . "./code.cg2js/cg-hotspot.fasl")
(#x86d050d32b951bfe67a8360001fa803a . "./code.cg2js/cg-html-dialog.fasl")
(#xdd6060a26bee8a86a56ec74443eb609a . "./code.cg2js/cg-html-widget.fasl")
(#x6447cf4e2069b82193cde7e8adcbabe9 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x42341d9567868da25496445c0ec2f259 . "./code.cg2js/cg-icon.fasl")
(#x7c0119b26509899762eb2e00c9bf897c . "./code.cg2js/cg-ie.fasl")
(#x2e5b09e8d08acb1cb42e43a33d711f24 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xf9de8f2619121b8cca286e9d3dead82b . "./code.cg2js/cg-intersect.fasl")
(#x38965285c26cdf3781c3ff6a22c5f4cc . "./code.cg2js/cg-item-list.fasl")
(#x5b7614a2b8e33f307b0a58fe41d4e967 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x44c232b69ddac2fdf17c7bd20820cfac . "./code.cg2js/cg-lamp.fasl")
(#x8cdb788f5ced1dfd103cf8c21294070b . "./code.cg2js/cg-layout.fasl")
(#x460131ecf1257ba5c4a99e4db29c4e45 . "./code.cg2js/cg-lettered-menu.fasl")
(#x1086799cdd0080993430bb0d364eb986 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x98f8429e5357fe6c9dde59540210ec22 . "./code.cg2js/cg-lisp-text.fasl")
(#x11026cf3715855fd80a23b550c1a6e90 . "./code.cg2js/cg-lisp-widget.fasl")
(#xbfe232400a11ac31d3ccabfe9c4d90a3 . "./code.cg2js/cg-list-view.fasl")
(#x5774aaa9c09a31618dcd4a730e0901f7 . "./code.cg2js/cg-mci.fasl")
(#x4a95f5c239cacaddc75a7785a18ef242 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x3a4e80fcf6053cc2182202b9188e28bb . "./code.cg2js/cg-menu.fasl")
(#x96ff5029bb7a37267618ab1b20688754 . "./code.cg2js/cg-message-dialog.fasl")
(#x5929e97afb71fcc53d1d8b1e8b2ab1d6 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x9b22cdd8fe459a8d28f8dcd19324c74c . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xa7e8a1a03c447d643f8a1016494b3bfe . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x066db013b003fbe0d7fa1e5fd63ed5da . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x0c7bc10b6693ad6b5636af8b8d9a9151 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x16da181dceb25a1dfba92aa036ba73cf . "./code.cg2js/cg-nodes.fasl")
(#xa4d17f168344a340e5d2290fc9596764 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x1b769e2eb2290c35d39c0f460fbf5f30 . "./code.cg2js/cg-object-editor.fasl")
(#xe27c7efdcc8fb526a06cc25e4b05d898 . "./code.cg2js/cg-ocx.fasl")
(#xaad7ecca4e4ce9f23071a89a66849e49 . "./code.cg2js/cg-os-widget.fasl")
(#x6374749a87aa4def873943852d36f655 . "./code.cg2js/cg-os-window.fasl")
(#xc72d52b0d9d8259c5f09ceb3474c4401 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xc379330c38a6348cd9b8251f1474628c . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x7fb22b9c0d3c78f475ec33b9624e3034 . "./code.cg2js/cg-outline.fasl")
(#xf650714938574fd5c8b018a05f864324 . "./code.cg2js/cg-palette.fasl")
(#x4c740bce12aa499dde1843e2f299ae51 . "./code.cg2js/cg-paren-matching.fasl")
(#x6036535bba1e27791d3165b4656259b7 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x1ba8cf05c4093c401395a47808eb2465 . "./code.cg2js/cg-picture-widget.fasl")
(#x8d47575953f014181b26647993b63b2d . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xa773c7b82c44ef29a076119aeea70222 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x98c8e3aa27cd12babc81ab29f1666ad4 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x63588f04c459dcec944b90c698457ebc . "./code.cg2js/cg-pixmap-widget.fasl")
(#x59749db6f63f76fe1c79cc9b6491bd81 . "./code.cg2js/cg-pixmap.fasl")
(#x134563fd65362dcc0bdb218584eca0bf . "./code.cg2js/cg-plot-widget.fasl")
(#xbdfd3855f1930a6d1ad959037c7c2bdc . "./code.cg2js/cg-printing.fasl")
(#xf2e93e558a28898c8a71d23f5d3cdba6 . "./code.cg2js/cg-progress-indicator.fasl")
(#x89ada27fd297831d74b968311ed20199 . "./code.cg2js/cg-project-window.fasl")
(#xe3bc81cd5397784ddd7faff4bcf92431 . "./code.cg2js/cg-property.fasl")
(#x017c3d80228c5c433cba63bbd55888fa . "./code.cg2js/cg-radio-button.fasl")
(#x43772cd3bab7f1ec23a81cd6e5724dd7 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x9f339b0a8b95bd9ee28e80ddce62154e . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xca35c33c4d7f386fa9b893b304974c3d . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x51964f06873e090435b597e1a6159bdc . "./code.cg2js/cg-rich-edit.fasl")
(#x64d8030b3ecd3b99b390910318eeae9e . "./code.cg2js/cg-sample-file-menu.fasl")
(#x3878c55b11319a914f4996a6ca7c93a3 . "./code.cg2js/cg-scaling-stream.fasl")
(#x96099a1eb0a8ed7a010a33ec408cf80a . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x5052430a7649241dea99d87d63ac4652 . "./code.cg2js/cg-scroll-bar.fasl")
(#xb4fce359a92060201a3b1839335c2566 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd954fe3a0bd19ce6d1da506e3d92e43a . "./code.cg2js/cg-selected-object.fasl")
(#x25874093ffa92891263c78d70b133640 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xb4e7e597c24d2a20344e88492116479c . "./code.cg2js/cg-split-bar.fasl")
(#x8d054e101beaf85c717f7cc8f44ebd2e . "./code.cg2js/cg-static-text.fasl")
(#x62aa33ea46ef483f5af01c28bf7be443 . "./code.cg2js/cg-status-bar.fasl")
(#xa6adee29daa87c93fe8b302769499a40 . "./code.cg2js/cg-string-dialog.fasl")
(#xa37abbc20bf199799c11a451df64d6a3 . "./code.cg2js/cg-tab-control.fasl")
(#x8e4b8744c679705b5d3aeca803e7613f . "./code.cg2js/cg-template-string.fasl")
(#xdcea7ef133fa77fb10f3aaec469bbb0f . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x85d02512d5d3bdb847d750f6c7145b30 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xb4864cf0c89aea15cc1f096d8ff7ac1c . "./code.cg2js/cg-text-edit-pane.fasl")
(#x3260633564ba34dc09754138548d368d . "./code.cg2js/cg-text-or-combo.fasl")
(#x7cb269c1a9588a76645ff0482fb2d2d0 . "./code.cg2js/cg-text-widget.fasl")
(#x199d66f0fcf62a62b810f6ba038b1e18 . "./code.cg2js/cg-timer.fasl")
(#xb3cd7e12d3e10aa615564d27bd84b001 . "./code.cg2js/cg-toggling-widget.fasl")
(#x2b7da843d2e9a8f2f02b252c9aaf23a8 . "./code.cg2js/cg-toolbar.fasl")
(#x2b1da5c8136b5eb6674e80077742553e . "./code.cg2js/cg-tooltip.fasl")
(#x52b29651ac98b26cb4969460c73d168f . "./code.cg2js/cg-touch.fasl")
(#xf428cf28a6c04aabf84a4156a13a9f25 . "./code.cg2js/cg-trackbar.fasl")
(#xc6ad9ec33b7e0aceb768eae0dec887f4 . "./code.cg2js/cg-tray.fasl")
(#x7aaef06322506424c807df7930ce557c . "./code.cg2js/cg-up-down-control.fasl")
(#xba53187bc9e405b1c545f104195e801c . "./code.cg2js/cg-utility-dialog.fasl")
(#x796089640036c760abf9c5404028bc1a . "./code.cg2js/cg-web-browser-dde.fasl")
(#xa3e7bd6b269579be91a4d113ed4fa605 . "./code.cg2js/cg-web-browser.fasl")
(#xdffd070ed70bfa640ef75ea8e5ec0940 . "./code.cg2js/cg-wrap-string.fasl")
(#x0281c52643d301a6f85fca9bb0f101a4 . "./code.cg2js/cg-yes-no-list.fasl")
(#x0d00f44a2331f08066b00890e940d4c8 . "./code.cg2js/cg-yes-no-string.fasl")
(#x3229c8f1c94070a5174da32851c2bb06 . "./code.cg2js/cg.fasl")
(#xc8c747cd088be5496ad2e80594b199ed . "./code.idejs/ide-acache.fasl")
(#xf969b08b86113a45d4f72ac3ba6efdc6 . "./code.idejs/ide-apropos.fasl")
(#xd2e28007ed318a1f4aa579ed299794cb . "./code.idejs/ide-base.fasl")
(#xd82079a6a5bcdf458f596200c33a850b . "./code.idejs/ide-class-browser.fasl")
(#x1d3825f292373889277380206172d8c2 . "./code.idejs/ide-class-grapher.fasl")
(#xf78bea5d6d94684ec11a56eb1edc7271 . "./code.idejs/ide-clipboard.fasl")
(#x4209fc0ff76db69e9a2cfc15e6622917 . "./code.idejs/ide-dev.fasl")
(#x783c2dd5cd645fb4f5f23dd45bfe8fd8 . "./code.idejs/ide-editor.fasl")
(#x70c30a67ea639f47139aeb9fd76bc6b0 . "./code.idejs/ide-find-definitions.fasl")
(#xacec91dfb849317380fbc2b5e9b624e7 . "./code.idejs/ide-find-in-files.fasl")
(#xee7c9cdc5dceb2f52d3839df799c51b9 . "./code.idejs/ide-form.fasl")
(#x1c198b374f339d248384d131042e041b . "./code.idejs/ide-grapher.fasl")
(#x8d28ce368b553b93ceb8fd86f0bc44d2 . "./code.idejs/ide-inspector.fasl")
(#x19303a1a33baff67a22ea67af2f23afe . "./code.idejs/ide-intro.fasl")
(#xc4d47c850c36944f888fc6c814e6a2bd . "./code.idejs/ide-keyshort.fasl")
(#x7a348399e06b0525025788dd0955a196 . "./code.idejs/ide-navigator.fasl")
(#x78cb1b7d7cc87b1d8136f97b484b3a3c . "./code.idejs/ide-package-list.fasl")
(#x3f34fc33c242a6d86ada69d3cbd15157 . "./code.idejs/ide-patch-dialog.fasl")
(#x0f2fd2b0951bc9b1f65ae055aca98dc0 . "./code.idejs/ide-process-browser.fasl")
(#x1b4337050d1f851b6401f2d0060fb824 . "./code.idejs/ide-project.fasl")
(#x1b77fcea92ac6783d3a0096040aa0529 . "./code.idejs/ide-trace.fasl")
(#xa25c5b86562d9163d1010e8a9f5eff43 . "./code.idejs/ide-window-list.fasl")
(#xecdfc55131347052ccb615c6b0f9398b . "./code.idejs/ide.fasl")
(#xf456f3c7874f303a35ad67e1f3673be9 . "./cgjs.html")
(#x85aa0e1902236a67f67453f2c909ea34 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
